import fs_implementation from 'node:fs';
import process from 'node:process';
import path from 'node:path';
import { randomUUID, createHash } from 'node:crypto';
import os from 'node:os';

function sync_impl$g$2$1(fs, path, mode) {
	fs.chmodSync(path, mode);
}

async function async_impl$g$2$1(fs, path, mode) {
	await fs.promises.chmod(path, mode);
}

var chmod$2$1 = {sync_impl: sync_impl$g$2$1, async_impl: async_impl$g$2$1};

function sync_impl$f$2$1(fs, path, uid, gid) {
	fs.chownSync(path, uid, gid);
}

async function async_impl$f$2$1(fs, path, uid, gid) {
	await fs.promises.chown(path, uid, gid);
}

var chown$2$1 = {sync_impl: sync_impl$f$2$1, async_impl: async_impl$f$2$1};

function sync_impl$e$2$1(fs, src, dest) {
	fs.copyFileSync(src, dest);
}

async function async_impl$e$2$1(fs, src, dest) {
	await fs.promises.copyFile(src, dest);
}

var copyFile$4$1 = {sync_impl: sync_impl$e$2$1, async_impl: async_impl$e$2$1};

function sync_impl$d$2$1(fs, path) {
	return fs.lstatSync(path)
}

async function async_impl$d$2$1(fs, path) {
	return await fs.promises.lstat(path)
}

var lstat$2$1 = {sync_impl: sync_impl$d$2$1, async_impl: async_impl$d$2$1};

function sync_impl$c$2$1(fs, path, options = null) {
	if (options === null) {
		fs.mkdirSync(path);
	} else {
		fs.mkdirSync(path, options);
	}
}

async function async_impl$c$2$1(fs, path, options = null) {
	if (options === null) {
		await fs.promises.mkdir(path);
	} else {
		await fs.promises.mkdir(path, options);
	}
}

var mkdir$2$1 = {sync_impl: sync_impl$c$2$1, async_impl: async_impl$c$2$1};

function sync_impl$b$2$1(fs, path) {
	const handle = fs.opendirSync(path);

	return {
		read() {
			return handle.readSync()
		},
		close() {
			handle.closeSync();
		}
	}
}

async function async_impl$b$2$1(fs, path) {
	const handle = await fs.promises.opendir(path);

	return {
		async read() {
			return await handle.read()
		},
		async close() {
			await handle.close();
		}
	}
}

var openDirectory$2$1 = {sync_impl: sync_impl$b$2$1, async_impl: async_impl$b$2$1};

function sync_impl$a$2$1(fs, path, flags = "r", mode = 0o666) {
	const fd = fs.openSync(path, flags, mode);

	return {
		read(buffer) {
			return fs.readSync(fd, buffer)
		},
		write(buffer) {
			return fs.writeSync(fd, buffer)
		},
		close() {
			fs.closeSync(fd);
		}
	}
}

async function async_impl$a$2$1(fs, path, flags = "r", mode = 0o666) {
	const handle = await fs.promises.open(path, flags, mode);

	return {
		async read(buffer) {
			const {bytesRead} = await handle.read(buffer, 0, buffer.byteLength, null);

			return bytesRead
		},
		async write(buffer) {
			const {bytesWritten} = await handle.write(buffer);

			return bytesWritten
		},
		async close() {
			await handle.close();
		}
	}
}

var openFile$2$1 = {sync_impl: sync_impl$a$2$1, async_impl: async_impl$a$2$1};

function sync_impl$9$2$1(fs, path) {
	return fs.readFileSync(path)
}

async function async_impl$9$2$1(fs, path) {
	return await fs.promises.readFile(path)
}

var readFile$2$1 = {sync_impl: sync_impl$9$2$1, async_impl: async_impl$9$2$1};

function sync_impl$8$2$1(fs, path) {
	return fs.readdirSync(path)
}

async function async_impl$8$2$1(fs, path) {
	return await fs.promises.readdir(path)
}

var readdir$2$1 = {sync_impl: sync_impl$8$2$1, async_impl: async_impl$8$2$1};

function sync_impl$7$2$1(fs, path) {
	return fs.readlinkSync(path)
}

async function async_impl$7$2$1(fs, path) {
	return await fs.promises.readlink(path)
}

var readlink$2$1 = {sync_impl: sync_impl$7$2$1, async_impl: async_impl$7$2$1};

function sync_impl$6$2$1(fs, path) {
	return fs.realpathSync(path)
}

async function async_impl$6$2$1(fs, path) {
	return await fs.promises.realpath(path)
}

var realpath$2$1 = {sync_impl: sync_impl$6$2$1, async_impl: async_impl$6$2$1};

function sync_impl$5$2$1(fs, src, dest) {
	fs.renameSync(src, dest);
}

async function async_impl$5$2$1(fs, src, dest) {
	await fs.promises.rename(src, dest);
}

var rename$2$1 = {sync_impl: sync_impl$5$2$1, async_impl: async_impl$5$2$1};

function sync_impl$4$2$1(fs, path) {
	fs.rmdirSync(path);
}

async function async_impl$4$2$1(fs, path) {
	await fs.promises.rmdir(path);
}

var rmdir$2$1 = {sync_impl: sync_impl$4$2$1, async_impl: async_impl$4$2$1};

function sync_impl$3$2$1(fs, path) {
	return fs.statSync(path)
}

async function async_impl$3$2$1(fs, path) {
	return await fs.promises.stat(path)
}

var stat$2$1 = {sync_impl: sync_impl$3$2$1, async_impl: async_impl$3$2$1};

function sync_impl$2$2$1(fs, target, path) {
	fs.symlinkSync(target, path);
}

async function async_impl$2$2$1(fs, target, path) {
	await fs.promises.symlink(target, path);
}

var symlink$2$1 = {sync_impl: sync_impl$2$2$1, async_impl: async_impl$2$2$1};

function sync_impl$1$2$1(fs, path) {
	fs.unlinkSync(path);
}

async function async_impl$1$2$1(fs, path) {
	await fs.promises.unlink(path);
}

var unlink$2$1 = {sync_impl: sync_impl$1$2$1, async_impl: async_impl$1$2$1};

function sync_impl$k$1(fs, path) {
	return fs.writeFileSync(path)
}

async function async_impl$k$1(fs, path) {
	return await fs.promises.writeFile(path)
}

var writeFile$2$1 = {sync_impl: sync_impl$k$1, async_impl: async_impl$k$1};

/* warning: this file was automatically generated */

var methods$2$1 = {
    chmod: chmod$2$1,
    chown: chown$2$1,
    copyFile: copyFile$4$1,
    lstat: lstat$2$1,
    mkdir: mkdir$2$1,
    openDirectory: openDirectory$2$1,
    openFile: openFile$2$1,
    readFile: readFile$2$1,
    readdir: readdir$2$1,
    readlink: readlink$2$1,
    realpath: realpath$2$1,
    rename: rename$2$1,
    rmdir: rmdir$2$1,
    stat: stat$2$1,
    symlink: symlink$2$1,
    unlink: unlink$2$1,
    writeFile: writeFile$2$1,
};

function createFSObject$2$1({
	sync = false
} = {}) {
	let fs_object = {};

	for (const method_name in methods$2$1) {
		const method = methods$2$1[method_name];
		const method_implementation = sync ? method.sync_impl : method.async_impl;

		fs_object[method_name] = (...args) => {

			if (process.env) {
				if ("ANIO_NODE_FS_API_DEBUG" in process.env) {
					console.error(`anio-node-fs-api ${method_name}${sync ? "Sync" : "Async"} with args`, args);
				}
			}

			return method_implementation(fs_implementation, ...args)
		};
	}

	return fs_object
}

function sync_impl$g$1$1(fs, path, mode) {
	fs.chmodSync(path, mode);
}

async function async_impl$g$1$1(fs, path, mode) {
	await fs.promises.chmod(path, mode);
}

var chmod$1$1 = {sync_impl: sync_impl$g$1$1, async_impl: async_impl$g$1$1};

function sync_impl$f$1$1(fs, path, uid, gid) {
	fs.chownSync(path, uid, gid);
}

async function async_impl$f$1$1(fs, path, uid, gid) {
	await fs.promises.chown(path, uid, gid);
}

var chown$1$1 = {sync_impl: sync_impl$f$1$1, async_impl: async_impl$f$1$1};

function sync_impl$e$1$1(fs, src, dest) {
	fs.copyFileSync(src, dest);
}

async function async_impl$e$1$1(fs, src, dest) {
	await fs.promises.copyFile(src, dest);
}

var copyFile$3$1 = {sync_impl: sync_impl$e$1$1, async_impl: async_impl$e$1$1};

function sync_impl$d$1$1(fs, path) {
	return fs.lstatSync(path)
}

async function async_impl$d$1$1(fs, path) {
	return await fs.promises.lstat(path)
}

var lstat$1$1 = {sync_impl: sync_impl$d$1$1, async_impl: async_impl$d$1$1};

function sync_impl$c$1$1(fs, path, options = null) {
	if (options === null) {
		fs.mkdirSync(path);
	} else {
		fs.mkdirSync(path, options);
	}
}

async function async_impl$c$1$1(fs, path, options = null) {
	if (options === null) {
		await fs.promises.mkdir(path);
	} else {
		await fs.promises.mkdir(path, options);
	}
}

var mkdir$1$1 = {sync_impl: sync_impl$c$1$1, async_impl: async_impl$c$1$1};

function sync_impl$b$1$1(fs, path) {
	const handle = fs.opendirSync(path);

	return {
		read() {
			return handle.readSync()
		},
		close() {
			handle.closeSync();
		}
	}
}

async function async_impl$b$1$1(fs, path) {
	const handle = await fs.promises.opendir(path);

	return {
		async read() {
			return await handle.read()
		},
		async close() {
			await handle.close();
		}
	}
}

var openDirectory$1$1 = {sync_impl: sync_impl$b$1$1, async_impl: async_impl$b$1$1};

function sync_impl$a$1$1(fs, path, flags = "r", mode = 0o666) {
	const fd = fs.openSync(path, flags, mode);

	return {
		read(buffer) {
			return fs.readSync(fd, buffer)
		},
		write(buffer) {
			return fs.writeSync(fd, buffer)
		},
		close() {
			fs.closeSync(fd);
		}
	}
}

async function async_impl$a$1$1(fs, path, flags = "r", mode = 0o666) {
	const handle = await fs.promises.open(path, flags, mode);

	return {
		async read(buffer) {
			const {bytesRead} = await handle.read(buffer, 0, buffer.byteLength, null);

			return bytesRead
		},
		async write(buffer) {
			const {bytesWritten} = await handle.write(buffer);

			return bytesWritten
		},
		async close() {
			await handle.close();
		}
	}
}

var openFile$1$1 = {sync_impl: sync_impl$a$1$1, async_impl: async_impl$a$1$1};

function sync_impl$9$1$1(fs, path) {
	return fs.readFileSync(path)
}

async function async_impl$9$1$1(fs, path) {
	return await fs.promises.readFile(path)
}

var readFile$1$1 = {sync_impl: sync_impl$9$1$1, async_impl: async_impl$9$1$1};

function sync_impl$8$1$1(fs, path) {
	return fs.readdirSync(path)
}

async function async_impl$8$1$1(fs, path) {
	return await fs.promises.readdir(path)
}

var readdir$1$1 = {sync_impl: sync_impl$8$1$1, async_impl: async_impl$8$1$1};

function sync_impl$7$1$1(fs, path) {
	return fs.readlinkSync(path)
}

async function async_impl$7$1$1(fs, path) {
	return await fs.promises.readlink(path)
}

var readlink$1$1 = {sync_impl: sync_impl$7$1$1, async_impl: async_impl$7$1$1};

function sync_impl$6$1$1(fs, path) {
	return fs.realpathSync(path)
}

async function async_impl$6$1$1(fs, path) {
	return await fs.promises.realpath(path)
}

var realpath$1$1 = {sync_impl: sync_impl$6$1$1, async_impl: async_impl$6$1$1};

function sync_impl$5$1$1(fs, src, dest) {
	fs.renameSync(src, dest);
}

async function async_impl$5$1$1(fs, src, dest) {
	await fs.promises.rename(src, dest);
}

var rename$1$1 = {sync_impl: sync_impl$5$1$1, async_impl: async_impl$5$1$1};

function sync_impl$4$1$1(fs, path) {
	fs.rmdirSync(path);
}

async function async_impl$4$1$1(fs, path) {
	await fs.promises.rmdir(path);
}

var rmdir$1$1 = {sync_impl: sync_impl$4$1$1, async_impl: async_impl$4$1$1};

function sync_impl$3$1$1(fs, path) {
	return fs.statSync(path)
}

async function async_impl$3$1$1(fs, path) {
	return await fs.promises.stat(path)
}

var stat$1$1 = {sync_impl: sync_impl$3$1$1, async_impl: async_impl$3$1$1};

function sync_impl$2$1$1(fs, target, path) {
	fs.symlinkSync(target, path);
}

async function async_impl$2$1$1(fs, target, path) {
	await fs.promises.symlink(target, path);
}

var symlink$1$1 = {sync_impl: sync_impl$2$1$1, async_impl: async_impl$2$1$1};

function sync_impl$1$1$1(fs, path) {
	fs.unlinkSync(path);
}

async function async_impl$1$1$1(fs, path) {
	await fs.promises.unlink(path);
}

var unlink$1$1 = {sync_impl: sync_impl$1$1$1, async_impl: async_impl$1$1$1};

function sync_impl$h$1$1(fs, path) {
	return fs.writeFileSync(path)
}

async function async_impl$h$1$1(fs, path) {
	return await fs.promises.writeFile(path)
}

var writeFile$1$1 = {sync_impl: sync_impl$h$1$1, async_impl: async_impl$h$1$1};

/* warning: this file was automatically generated */

var methods$1$1 = {
    chmod: chmod$1$1,
    chown: chown$1$1,
    copyFile: copyFile$3$1,
    lstat: lstat$1$1,
    mkdir: mkdir$1$1,
    openDirectory: openDirectory$1$1,
    openFile: openFile$1$1,
    readFile: readFile$1$1,
    readdir: readdir$1$1,
    readlink: readlink$1$1,
    realpath: realpath$1$1,
    rename: rename$1$1,
    rmdir: rmdir$1$1,
    stat: stat$1$1,
    symlink: symlink$1$1,
    unlink: unlink$1$1,
    writeFile: writeFile$1$1,
};

function createFSObject$1$1({
	sync = false
} = {}) {
	let fs_object = {};

	for (const method_name in methods$1$1) {
		const method = methods$1$1[method_name];
		const method_implementation = sync ? method.sync_impl : method.async_impl;

		fs_object[method_name] = (...args) => {

			if (process.env) {
				if ("ANIO_NODE_FS_API_DEBUG" in process.env) {
					console.error(`anio-node-fs-api ${method_name}${sync ? "Sync" : "Async"} with args`, args);
				}
			}

			return method_implementation(fs_implementation, ...args)
		};
	}

	return fs_object
}

createFSObject$1$1({sync: false});
createFSObject$1$1({sync: true});

function sync_impl$g$6(fs, path, mode) {
	fs.chmodSync(path, mode);
}

async function async_impl$g$6(fs, path, mode) {
	await fs.promises.chmod(path, mode);
}

var chmod$6 = {sync_impl: sync_impl$g$6, async_impl: async_impl$g$6};

function sync_impl$f$6(fs, path, uid, gid) {
	fs.chownSync(path, uid, gid);
}

async function async_impl$f$6(fs, path, uid, gid) {
	await fs.promises.chown(path, uid, gid);
}

var chown$6 = {sync_impl: sync_impl$f$6, async_impl: async_impl$f$6};

function sync_impl$e$6(fs, src, dest) {
	fs.copyFileSync(src, dest);
}

async function async_impl$e$6(fs, src, dest) {
	await fs.promises.copyFile(src, dest);
}

var copyFile$2$1 = {sync_impl: sync_impl$e$6, async_impl: async_impl$e$6};

function sync_impl$d$6(fs, path) {
	return fs.lstatSync(path)
}

async function async_impl$d$6(fs, path) {
	return await fs.promises.lstat(path)
}

var lstat$6 = {sync_impl: sync_impl$d$6, async_impl: async_impl$d$6};

function sync_impl$c$6(fs, path, options = null) {
	if (options === null) {
		fs.mkdirSync(path);
	} else {
		fs.mkdirSync(path, options);
	}
}

async function async_impl$c$6(fs, path, options = null) {
	if (options === null) {
		await fs.promises.mkdir(path);
	} else {
		await fs.promises.mkdir(path, options);
	}
}

var mkdir$6 = {sync_impl: sync_impl$c$6, async_impl: async_impl$c$6};

function sync_impl$b$6(fs, path) {
	const handle = fs.opendirSync(path);

	return {
		read() {
			return handle.readSync()
		},
		close() {
			handle.closeSync();
		}
	}
}

async function async_impl$b$6(fs, path) {
	const handle = await fs.promises.opendir(path);

	return {
		async read() {
			return await handle.read()
		},
		async close() {
			await handle.close();
		}
	}
}

var openDirectory$6 = {sync_impl: sync_impl$b$6, async_impl: async_impl$b$6};

function sync_impl$a$6(fs, path, flags = "r", mode = 0o666) {
	const fd = fs.openSync(path, flags, mode);

	return {
		read(buffer) {
			return fs.readSync(fd, buffer)
		},
		write(buffer) {
			return fs.writeSync(fd, buffer)
		},
		close() {
			fs.closeSync(fd);
		}
	}
}

async function async_impl$a$6(fs, path, flags = "r", mode = 0o666) {
	const handle = await fs.promises.open(path, flags, mode);

	return {
		async read(buffer) {
			const {bytesRead} = await handle.read(buffer, 0, buffer.byteLength, null);

			return bytesRead
		},
		async write(buffer) {
			const {bytesWritten} = await handle.write(buffer);

			return bytesWritten
		},
		async close() {
			await handle.close();
		}
	}
}

var openFile$6 = {sync_impl: sync_impl$a$6, async_impl: async_impl$a$6};

function sync_impl$9$6(fs, path) {
	return fs.readFileSync(path)
}

async function async_impl$9$6(fs, path) {
	return await fs.promises.readFile(path)
}

var readFile$5 = {sync_impl: sync_impl$9$6, async_impl: async_impl$9$6};

function sync_impl$8$6(fs, path) {
	return fs.readdirSync(path)
}

async function async_impl$8$6(fs, path) {
	return await fs.promises.readdir(path)
}

var readdir$6 = {sync_impl: sync_impl$8$6, async_impl: async_impl$8$6};

function sync_impl$7$6(fs, path) {
	return fs.readlinkSync(path)
}

async function async_impl$7$6(fs, path) {
	return await fs.promises.readlink(path)
}

var readlink$6 = {sync_impl: sync_impl$7$6, async_impl: async_impl$7$6};

function sync_impl$6$6(fs, path) {
	return fs.realpathSync(path)
}

async function async_impl$6$6(fs, path) {
	return await fs.promises.realpath(path)
}

var realpath$6 = {sync_impl: sync_impl$6$6, async_impl: async_impl$6$6};

function sync_impl$5$6(fs, src, dest) {
	fs.renameSync(src, dest);
}

async function async_impl$5$6(fs, src, dest) {
	await fs.promises.rename(src, dest);
}

var rename$6 = {sync_impl: sync_impl$5$6, async_impl: async_impl$5$6};

function sync_impl$4$6(fs, path) {
	fs.rmdirSync(path);
}

async function async_impl$4$6(fs, path) {
	await fs.promises.rmdir(path);
}

var rmdir$6 = {sync_impl: sync_impl$4$6, async_impl: async_impl$4$6};

function sync_impl$3$6(fs, path) {
	return fs.statSync(path)
}

async function async_impl$3$6(fs, path) {
	return await fs.promises.stat(path)
}

var stat$6 = {sync_impl: sync_impl$3$6, async_impl: async_impl$3$6};

function sync_impl$2$6(fs, target, path) {
	fs.symlinkSync(target, path);
}

async function async_impl$2$6(fs, target, path) {
	await fs.promises.symlink(target, path);
}

var symlink$6 = {sync_impl: sync_impl$2$6, async_impl: async_impl$2$6};

function sync_impl$1$6(fs, path) {
	fs.unlinkSync(path);
}

async function async_impl$1$6(fs, path) {
	await fs.promises.unlink(path);
}

var unlink$6 = {sync_impl: sync_impl$1$6, async_impl: async_impl$1$6};

function sync_impl$h$4(fs, path) {
	return fs.writeFileSync(path)
}

async function async_impl$h$4(fs, path) {
	return await fs.promises.writeFile(path)
}

var writeFile$5 = {sync_impl: sync_impl$h$4, async_impl: async_impl$h$4};

/* warning: this file was automatically generated */

var methods$6 = {
    chmod: chmod$6,
    chown: chown$6,
    copyFile: copyFile$2$1,
    lstat: lstat$6,
    mkdir: mkdir$6,
    openDirectory: openDirectory$6,
    openFile: openFile$6,
    readFile: readFile$5,
    readdir: readdir$6,
    readlink: readlink$6,
    realpath: realpath$6,
    rename: rename$6,
    rmdir: rmdir$6,
    stat: stat$6,
    symlink: symlink$6,
    unlink: unlink$6,
    writeFile: writeFile$5,
};

function createFSObject$6({
	sync = false
} = {}) {
	let fs_object = {};

	for (const method_name in methods$6) {
		const method = methods$6[method_name];
		const method_implementation = sync ? method.sync_impl : method.async_impl;

		fs_object[method_name] = (...args) => {

			if (process.env) {
				if ("ANIO_NODE_FS_API_DEBUG" in process.env) {
					console.error(`anio-node-fs-api ${method_name}${sync ? "Sync" : "Async"} with args`, args);
				}
			}

			return method_implementation(fs_implementation, ...args)
		};
	}

	return fs_object
}

createFSObject$6({sync: false});
createFSObject$6({sync: true});

createFSObject$2$1({sync: false});
createFSObject$2$1({sync: true});

async function copy(...args) {
	return await copy(...args)
}

copy.sync = function(...args) {
	return copy.sync(...args)
};

function generateTemporaryPathNameSync() {
	return path.join(os.tmpdir(), randomUUID())
}

async function generateTemporaryPathName() {
	return generateTemporaryPathNameSync()
}

generateTemporaryPathName.sync = function() {
	return generateTemporaryPathNameSync()
};

function sync_impl$e$5(fs, path, mode) {
	fs.chmodSync(path, mode);
}

async function async_impl$e$5(fs, path, mode) {
	await fs.promises.chmod(path, mode);
}

var chmod$5 = {sync_impl: sync_impl$e$5, async_impl: async_impl$e$5};

function sync_impl$d$5(fs, path, uid, gid) {
	fs.chownSync(path, uid, gid);
}

async function async_impl$d$5(fs, path, uid, gid) {
	await fs.promises.chown(path, uid, gid);
}

var chown$5 = {sync_impl: sync_impl$d$5, async_impl: async_impl$d$5};

function sync_impl$c$5(fs, src, dest) {
	fs.copyFileSync(src, dest);
}

async function async_impl$c$5(fs, src, dest) {
	await fs.promises.copyFile(src, dest);
}

var copyFile$5 = {sync_impl: sync_impl$c$5, async_impl: async_impl$c$5};

function sync_impl$b$5(fs, path) {
	return fs.lstatSync(path)
}

async function async_impl$b$5(fs, path) {
	return await fs.promises.lstat(path)
}

var lstat$5 = {sync_impl: sync_impl$b$5, async_impl: async_impl$b$5};

function sync_impl$a$5(fs, path) {
	fs.mkdirSync(path);
}

async function async_impl$a$5(fs, path) {
	await fs.promises.mkdir(path);
}

var mkdir$5 = {sync_impl: sync_impl$a$5, async_impl: async_impl$a$5};

function sync_impl$9$5(fs, path) {
	const handle = fs.opendirSync(path);

	return {
		read() {
			return handle.readSync()
		},
		close() {
			handle.closeSync();
		}
	}
}

async function async_impl$9$5(fs, path) {
	const handle = await fs.promises.opendir(path);

	return {
		async read() {
			return await handle.read()
		},
		async close() {
			await handle.close();
		}
	}
}

var openDirectory$5 = {sync_impl: sync_impl$9$5, async_impl: async_impl$9$5};

function sync_impl$8$5(fs, path, flags = "r", mode = 0o666) {
	const fd = fs.openSync(path, flags, mode);

	return {
		read(buffer) {
			return fs.readSync(fd, buffer)
		},
		write(buffer) {
			return fs.writeSync(fd, buffer)
		},
		close() {
			fs.closeSync(fd);
		}
	}
}

async function async_impl$8$5(fs, path, flags = "r", mode = 0o666) {
	const handle = await fs.promises.open(path, flags, mode);

	return {
		async read(buffer) {
			const {bytesRead} = await handle.read(buffer, 0, buffer.byteLength, null);

			return bytesRead
		},
		async write(buffer) {
			const {bytesWritten} = await handle.write(buffer);

			return bytesWritten
		},
		async close() {
			await handle.close();
		}
	}
}

var openFile$5 = {sync_impl: sync_impl$8$5, async_impl: async_impl$8$5};

function sync_impl$7$5(fs, path) {
	return fs.readdirSync(path)
}

async function async_impl$7$5(fs, path) {
	return await fs.promises.readdir(path)
}

var readdir$5 = {sync_impl: sync_impl$7$5, async_impl: async_impl$7$5};

function sync_impl$6$5(fs, path) {
	return fs.readlinkSync(path)
}

async function async_impl$6$5(fs, path) {
	return await fs.promises.readlink(path)
}

var readlink$5 = {sync_impl: sync_impl$6$5, async_impl: async_impl$6$5};

function sync_impl$5$5(fs, path) {
	return fs.realpathSync(path)
}

async function async_impl$5$5(fs, path) {
	return await fs.promises.realpath(path)
}

var realpath$5 = {sync_impl: sync_impl$5$5, async_impl: async_impl$5$5};

function sync_impl$4$5(fs, src, dest) {
	fs.renameSync(src, dest);
}

async function async_impl$4$5(fs, src, dest) {
	await fs.promises.rename(src, dest);
}

var rename$5 = {sync_impl: sync_impl$4$5, async_impl: async_impl$4$5};

function sync_impl$3$5(fs, path) {
	fs.rmdirSync(path);
}

async function async_impl$3$5(fs, path) {
	await fs.promises.rmdir(path);
}

var rmdir$5 = {sync_impl: sync_impl$3$5, async_impl: async_impl$3$5};

function sync_impl$2$5(fs, path) {
	return fs.statSync(path)
}

async function async_impl$2$5(fs, path) {
	return await fs.promises.stat(path)
}

var stat$5 = {sync_impl: sync_impl$2$5, async_impl: async_impl$2$5};

function sync_impl$1$5(fs, target, path) {
	fs.symlinkSync(target, path);
}

async function async_impl$1$5(fs, target, path) {
	await fs.promises.symlink(target, path);
}

var symlink$5 = {sync_impl: sync_impl$1$5, async_impl: async_impl$1$5};

function sync_impl$f$5(fs, path) {
	fs.unlinkSync(path);
}

async function async_impl$f$5(fs, path) {
	await fs.promises.unlink(path);
}

var unlink$5 = {sync_impl: sync_impl$f$5, async_impl: async_impl$f$5};

/* warning: this file was automatically generated */

var methods$5 = {
    chmod: chmod$5,
    chown: chown$5,
    copyFile: copyFile$5,
    lstat: lstat$5,
    mkdir: mkdir$5,
    openDirectory: openDirectory$5,
    openFile: openFile$5,
    readdir: readdir$5,
    readlink: readlink$5,
    realpath: realpath$5,
    rename: rename$5,
    rmdir: rmdir$5,
    stat: stat$5,
    symlink: symlink$5,
    unlink: unlink$5,
};

function createFSObject$5({
	sync = false
} = {}) {
	let fs_object = {};

	for (const method_name in methods$5) {
		const method = methods$5[method_name];
		const method_implementation = sync ? method.sync_impl : method.async_impl;

		fs_object[method_name] = (...args) => {
			return method_implementation(fs_implementation, ...args)
		};
	}

	return fs_object
}

function sync_impl$g$5(fs_object, file_path, chunk_size) {
	const file = fs_object.openFile(file_path, "r");
	let closed = false;

	return {
		readNextChunk() {
			if (closed) return false

			let chunk_buffer = Buffer.alloc(chunk_size);

			const bytes_read = file.read(chunk_buffer);

			if (!bytes_read) {
				if (!closed) file.close();

				closed = true;

				return false
			}

			return chunk_buffer.subarray(0, bytes_read)
		},

		close() {
			if (closed) return

			file.close();
			closed = true;
		}
	}
}

async function async_impl$g$5(fs_object, file_path, chunk_size) {
	const file = await fs_object.openFile(file_path, "r");
	let closed = false;

	return {
		async readNextChunk() {
			if (closed) return false

			let chunk_buffer = Buffer.alloc(chunk_size);

			const bytes_read = await file.read(chunk_buffer);

			if (!bytes_read) {
				if (!closed) await file.close();

				closed = true;

				return false
			}

			return chunk_buffer.subarray(0, bytes_read)
		},

		async close() {
			if (closed) return

			await file.close();
			closed = true;
		}
	}
}

const async_fs$5 = createFSObject$5({sync: false});
const sync_fs$5 = createFSObject$5({sync: true});

function nodeFsReadFileInChunks(path, chunk_size) {
	return async_impl$g$5(async_fs$5, path, chunk_size)
}

nodeFsReadFileInChunks.sync = function(path, chunk_size) {
	return sync_impl$g$5(sync_fs$5, path, chunk_size)
};

async function async_impl$n(file_path, algo, hash_encoding = "hex") {
	const hash = createHash(algo);

	//
	// read file in 4096 byte chunks
	//
	const file = await nodeFsReadFileInChunks(file_path, 4096);

	while (true) {
		const chunk = await file.readNextChunk();

		if (!chunk) break

		hash.update(chunk);
	}

	hash.setEncoding(hash_encoding);
	hash.end();

	return hash.read()
}

function sync_impl$n(file_path, algo, hash_encoding = "hex") {
	const hash = createHash(algo);

	//
	// read file in 4096 byte chunks
	//
	const file = nodeFsReadFileInChunks.sync(file_path, 4096);

	while (true) {
		const chunk = file.readNextChunk();

		if (!chunk) break

		hash.update(chunk);
	}

	hash.setEncoding(hash_encoding);
	hash.end();

	return hash.read()
}

function nodeFsHashFile(file_path, algo, hash_encoding = "hex") {
	return async_impl$n(file_path, algo, hash_encoding)
}

nodeFsHashFile.sync = function(file_path, algo, hash_encoding = "hex") {
	return sync_impl$n(file_path, algo, hash_encoding)
};

async function hashFile(...args) {
	return await nodeFsHashFile(...args)
}

hashFile.sync = function(...args) {
	return nodeFsHashFile.sync(...args)
};

function sync_impl$g$4(fs, path, mode) {
	fs.chmodSync(path, mode);
}

async function async_impl$g$4(fs, path, mode) {
	await fs.promises.chmod(path, mode);
}

var chmod$4 = {sync_impl: sync_impl$g$4, async_impl: async_impl$g$4};

function sync_impl$f$4(fs, path, uid, gid) {
	fs.chownSync(path, uid, gid);
}

async function async_impl$f$4(fs, path, uid, gid) {
	await fs.promises.chown(path, uid, gid);
}

var chown$4 = {sync_impl: sync_impl$f$4, async_impl: async_impl$f$4};

function sync_impl$e$4(fs, src, dest) {
	fs.copyFileSync(src, dest);
}

async function async_impl$e$4(fs, src, dest) {
	await fs.promises.copyFile(src, dest);
}

var copyFile$4 = {sync_impl: sync_impl$e$4, async_impl: async_impl$e$4};

function sync_impl$d$4(fs, path) {
	return fs.lstatSync(path)
}

async function async_impl$d$4(fs, path) {
	return await fs.promises.lstat(path)
}

var lstat$4 = {sync_impl: sync_impl$d$4, async_impl: async_impl$d$4};

function sync_impl$c$4(fs, path, options = null) {
	if (options === null) {
		fs.mkdirSync(path);
	} else {
		fs.mkdirSync(path, options);
	}
}

async function async_impl$c$4(fs, path, options = null) {
	if (options === null) {
		await fs.promises.mkdir(path);
	} else {
		await fs.promises.mkdir(path, options);
	}
}

var mkdir$4 = {sync_impl: sync_impl$c$4, async_impl: async_impl$c$4};

function sync_impl$b$4(fs, path) {
	const handle = fs.opendirSync(path);

	return {
		read() {
			return handle.readSync()
		},
		close() {
			handle.closeSync();
		}
	}
}

async function async_impl$b$4(fs, path) {
	const handle = await fs.promises.opendir(path);

	return {
		async read() {
			return await handle.read()
		},
		async close() {
			await handle.close();
		}
	}
}

var openDirectory$4 = {sync_impl: sync_impl$b$4, async_impl: async_impl$b$4};

function sync_impl$a$4(fs, path, flags = "r", mode = 0o666) {
	const fd = fs.openSync(path, flags, mode);

	return {
		read(buffer) {
			return fs.readSync(fd, buffer)
		},
		write(buffer) {
			return fs.writeSync(fd, buffer)
		},
		close() {
			fs.closeSync(fd);
		}
	}
}

async function async_impl$a$4(fs, path, flags = "r", mode = 0o666) {
	const handle = await fs.promises.open(path, flags, mode);

	return {
		async read(buffer) {
			const {bytesRead} = await handle.read(buffer, 0, buffer.byteLength, null);

			return bytesRead
		},
		async write(buffer) {
			const {bytesWritten} = await handle.write(buffer);

			return bytesWritten
		},
		async close() {
			await handle.close();
		}
	}
}

var openFile$4 = {sync_impl: sync_impl$a$4, async_impl: async_impl$a$4};

function sync_impl$9$4(fs, path) {
	return fs.readFileSync(path)
}

async function async_impl$9$4(fs, path) {
	return await fs.promises.readFile(path)
}

var readFile$4 = {sync_impl: sync_impl$9$4, async_impl: async_impl$9$4};

function sync_impl$8$4(fs, path) {
	return fs.readdirSync(path)
}

async function async_impl$8$4(fs, path) {
	return await fs.promises.readdir(path)
}

var readdir$4 = {sync_impl: sync_impl$8$4, async_impl: async_impl$8$4};

function sync_impl$7$4(fs, path) {
	return fs.readlinkSync(path)
}

async function async_impl$7$4(fs, path) {
	return await fs.promises.readlink(path)
}

var readlink$4 = {sync_impl: sync_impl$7$4, async_impl: async_impl$7$4};

function sync_impl$6$4(fs, path) {
	return fs.realpathSync(path)
}

async function async_impl$6$4(fs, path) {
	return await fs.promises.realpath(path)
}

var realpath$4 = {sync_impl: sync_impl$6$4, async_impl: async_impl$6$4};

function sync_impl$5$4(fs, src, dest) {
	fs.renameSync(src, dest);
}

async function async_impl$5$4(fs, src, dest) {
	await fs.promises.rename(src, dest);
}

var rename$4 = {sync_impl: sync_impl$5$4, async_impl: async_impl$5$4};

function sync_impl$4$4(fs, path) {
	fs.rmdirSync(path);
}

async function async_impl$4$4(fs, path) {
	await fs.promises.rmdir(path);
}

var rmdir$4 = {sync_impl: sync_impl$4$4, async_impl: async_impl$4$4};

function sync_impl$3$4(fs, path) {
	return fs.statSync(path)
}

async function async_impl$3$4(fs, path) {
	return await fs.promises.stat(path)
}

var stat$4 = {sync_impl: sync_impl$3$4, async_impl: async_impl$3$4};

function sync_impl$2$4(fs, target, path) {
	fs.symlinkSync(target, path);
}

async function async_impl$2$4(fs, target, path) {
	await fs.promises.symlink(target, path);
}

var symlink$4 = {sync_impl: sync_impl$2$4, async_impl: async_impl$2$4};

function sync_impl$1$4(fs, path) {
	fs.unlinkSync(path);
}

async function async_impl$1$4(fs, path) {
	await fs.promises.unlink(path);
}

var unlink$4 = {sync_impl: sync_impl$1$4, async_impl: async_impl$1$4};

function sync_impl$h$3(fs, path) {
	return fs.writeFileSync(path)
}

async function async_impl$h$3(fs, path) {
	return await fs.promises.writeFile(path)
}

var writeFile$4 = {sync_impl: sync_impl$h$3, async_impl: async_impl$h$3};

/* warning: this file was automatically generated */

var methods$4 = {
    chmod: chmod$4,
    chown: chown$4,
    copyFile: copyFile$4,
    lstat: lstat$4,
    mkdir: mkdir$4,
    openDirectory: openDirectory$4,
    openFile: openFile$4,
    readFile: readFile$4,
    readdir: readdir$4,
    readlink: readlink$4,
    realpath: realpath$4,
    rename: rename$4,
    rmdir: rmdir$4,
    stat: stat$4,
    symlink: symlink$4,
    unlink: unlink$4,
    writeFile: writeFile$4,
};

function createFSObject$4({
	sync = false
} = {}) {
	let fs_object = {};

	for (const method_name in methods$4) {
		const method = methods$4[method_name];
		const method_implementation = sync ? method.sync_impl : method.async_impl;

		fs_object[method_name] = (...args) => {

			if (process.env) {
				if ("ANIO_NODE_FS_API_DEBUG" in process.env) {
					console.error(`anio-node-fs-api ${method_name}${sync ? "Sync" : "Async"} with args`, args);
				}
			}

			return method_implementation(fs_implementation, ...args)
		};
	}

	return fs_object
}

function tryStat$1$1(fs_object, path, fn = "lstat") {
	try {
		return fs_object[fn](path)
	} catch (error) {
		//
		// ENOENT means no such entry
		//
		if (error.code === "ENOENT") {
			return false
		}

		// re-throw every other error
		throw error
	}
}

function sync_impl$m(fs_object, ...args) {
	const path_to_check = path.join(...args);

	const lstat = tryStat$1$1(fs_object, path_to_check);

	if (lstat === false) return false

	if (lstat.isSymbolicLink()) {
		const stat = tryStat$1$1(fs_object, path_to_check, "stat");

		if (stat === false) return "link->broken"
		if (stat.isDirectory()) return "link->dir"

		return "link->file"
	}

	if (lstat.isDirectory()) return "dir"

	return "file"
}

async function tryStat$2(fs_object, path, fn = "lstat") {
	try {
		return await fs_object[fn](path)
	} catch (error) {
		//
		// ENOENT means no such entry
		//
		if (error.code === "ENOENT") {
			return false
		}

		// re-throw every other error
		throw error
	}
}

async function async_impl$m(fs_object, ...args) {
	const path_to_check = path.join(...args);

	const lstat = await tryStat$2(fs_object, path_to_check);

	if (lstat === false) return false

	if (lstat.isSymbolicLink()) {
		const stat = await tryStat$2(fs_object, path_to_check, "stat");

		if (stat === false) return "link->broken"
		if (stat.isDirectory()) return "link->dir"

		return "link->file"
	}

	if (lstat.isDirectory()) return "dir"

	return "file"
}

const async_fs$4 = createFSObject$4({sync: false});
const sync_fs$4 = createFSObject$4({sync: true});

function nodeFsGetPathType$1(...args) {
	return async_impl$m(async_fs$4, ...args)
}

nodeFsGetPathType$1.sync = function(...args) {
	return sync_impl$m(sync_fs$4, ...args)
};

function processResult$4(result) {
	return result === "dir" || result === "link->dir"
}

async function isDirectory(...args) {
	return processResult$4(await nodeFsGetPathType$1(...args))
}

isDirectory.sync = function(...args) {
	return processResult$4(nodeFsGetPathType$1.sync(...args))
};

function processResult$3(result) {
	return result === "file" || result === "link->file"
}

async function isFile(...args) {
	return processResult$3(await nodeFsGetPathType$1(...args))
}

isFile.sync = function(...args) {
	return processResult$3(nodeFsGetPathType$1.sync(...args))
};

function processResult$2(result) {
	return result === "dir"
}

async function isRegularDirectory(...args) {
	return processResult$2(await nodeFsGetPathType$1(...args))
}

isRegularDirectory.sync = function(...args) {
	return processResult$2(nodeFsGetPathType$1.sync(...args))
};

function processResult$1(result) {
	return result === "file"
}

async function isRegularFile(...args) {
	return processResult$1(await nodeFsGetPathType$1(...args))
}

isRegularFile.sync = function(...args) {
	return processResult$1(nodeFsGetPathType$1.sync(...args))
};

function processResult(result) {
	return result.startsWith("link")
}

async function isSymbolicLink(...args) {
	return processResult(await nodeFsGetPathType$1(...args))
}

isSymbolicLink.sync = function(...args) {
	return processResult(nodeFsGetPathType$1.sync(...args))
};

function sync_impl$g$2(fs, path, mode) {
	fs.chmodSync(path, mode);
}

async function async_impl$g$2(fs, path, mode) {
	await fs.promises.chmod(path, mode);
}

var chmod$2 = {sync_impl: sync_impl$g$2, async_impl: async_impl$g$2};

function sync_impl$f$2(fs, path, uid, gid) {
	fs.chownSync(path, uid, gid);
}

async function async_impl$f$2(fs, path, uid, gid) {
	await fs.promises.chown(path, uid, gid);
}

var chown$2 = {sync_impl: sync_impl$f$2, async_impl: async_impl$f$2};

function sync_impl$e$2(fs, src, dest) {
	fs.copyFileSync(src, dest);
}

async function async_impl$e$2(fs, src, dest) {
	await fs.promises.copyFile(src, dest);
}

var copyFile$2 = {sync_impl: sync_impl$e$2, async_impl: async_impl$e$2};

function sync_impl$d$2(fs, path) {
	return fs.lstatSync(path)
}

async function async_impl$d$2(fs, path) {
	return await fs.promises.lstat(path)
}

var lstat$2 = {sync_impl: sync_impl$d$2, async_impl: async_impl$d$2};

function sync_impl$c$2(fs, path, options = null) {
	if (options === null) {
		fs.mkdirSync(path);
	} else {
		fs.mkdirSync(path, options);
	}
}

async function async_impl$c$2(fs, path, options = null) {
	if (options === null) {
		await fs.promises.mkdir(path);
	} else {
		await fs.promises.mkdir(path, options);
	}
}

var mkdir$2 = {sync_impl: sync_impl$c$2, async_impl: async_impl$c$2};

function sync_impl$b$2(fs, path) {
	const handle = fs.opendirSync(path);

	return {
		read() {
			return handle.readSync()
		},
		close() {
			handle.closeSync();
		}
	}
}

async function async_impl$b$2(fs, path) {
	const handle = await fs.promises.opendir(path);

	return {
		async read() {
			return await handle.read()
		},
		async close() {
			await handle.close();
		}
	}
}

var openDirectory$2 = {sync_impl: sync_impl$b$2, async_impl: async_impl$b$2};

function sync_impl$a$2(fs, path, flags = "r", mode = 0o666) {
	const fd = fs.openSync(path, flags, mode);

	return {
		read(buffer) {
			return fs.readSync(fd, buffer)
		},
		write(buffer) {
			return fs.writeSync(fd, buffer)
		},
		close() {
			fs.closeSync(fd);
		}
	}
}

async function async_impl$a$2(fs, path, flags = "r", mode = 0o666) {
	const handle = await fs.promises.open(path, flags, mode);

	return {
		async read(buffer) {
			const {bytesRead} = await handle.read(buffer, 0, buffer.byteLength, null);

			return bytesRead
		},
		async write(buffer) {
			const {bytesWritten} = await handle.write(buffer);

			return bytesWritten
		},
		async close() {
			await handle.close();
		}
	}
}

var openFile$2 = {sync_impl: sync_impl$a$2, async_impl: async_impl$a$2};

function sync_impl$9$2(fs, path) {
	return fs.readFileSync(path)
}

async function async_impl$9$2(fs, path) {
	return await fs.promises.readFile(path)
}

var readFile$2 = {sync_impl: sync_impl$9$2, async_impl: async_impl$9$2};

function sync_impl$8$2(fs, path) {
	return fs.readdirSync(path)
}

async function async_impl$8$2(fs, path) {
	return await fs.promises.readdir(path)
}

var readdir$2 = {sync_impl: sync_impl$8$2, async_impl: async_impl$8$2};

function sync_impl$7$2(fs, path) {
	return fs.readlinkSync(path)
}

async function async_impl$7$2(fs, path) {
	return await fs.promises.readlink(path)
}

var readlink$2 = {sync_impl: sync_impl$7$2, async_impl: async_impl$7$2};

function sync_impl$6$2(fs, path) {
	return fs.realpathSync(path)
}

async function async_impl$6$2(fs, path) {
	return await fs.promises.realpath(path)
}

var realpath$2 = {sync_impl: sync_impl$6$2, async_impl: async_impl$6$2};

function sync_impl$5$2(fs, src, dest) {
	fs.renameSync(src, dest);
}

async function async_impl$5$2(fs, src, dest) {
	await fs.promises.rename(src, dest);
}

var rename$2 = {sync_impl: sync_impl$5$2, async_impl: async_impl$5$2};

function sync_impl$4$2(fs, path) {
	fs.rmdirSync(path);
}

async function async_impl$4$2(fs, path) {
	await fs.promises.rmdir(path);
}

var rmdir$2 = {sync_impl: sync_impl$4$2, async_impl: async_impl$4$2};

function sync_impl$3$2(fs, path) {
	return fs.statSync(path)
}

async function async_impl$3$2(fs, path) {
	return await fs.promises.stat(path)
}

var stat$2 = {sync_impl: sync_impl$3$2, async_impl: async_impl$3$2};

function sync_impl$2$2(fs, target, path) {
	fs.symlinkSync(target, path);
}

async function async_impl$2$2(fs, target, path) {
	await fs.promises.symlink(target, path);
}

var symlink$2 = {sync_impl: sync_impl$2$2, async_impl: async_impl$2$2};

function sync_impl$1$2(fs, path) {
	fs.unlinkSync(path);
}

async function async_impl$1$2(fs, path) {
	await fs.promises.unlink(path);
}

var unlink$2 = {sync_impl: sync_impl$1$2, async_impl: async_impl$1$2};

function sync_impl$k(fs, path) {
	return fs.writeFileSync(path)
}

async function async_impl$k(fs, path) {
	return await fs.promises.writeFile(path)
}

var writeFile$2 = {sync_impl: sync_impl$k, async_impl: async_impl$k};

/* warning: this file was automatically generated */

var methods$2 = {
    chmod: chmod$2,
    chown: chown$2,
    copyFile: copyFile$2,
    lstat: lstat$2,
    mkdir: mkdir$2,
    openDirectory: openDirectory$2,
    openFile: openFile$2,
    readFile: readFile$2,
    readdir: readdir$2,
    readlink: readlink$2,
    realpath: realpath$2,
    rename: rename$2,
    rmdir: rmdir$2,
    stat: stat$2,
    symlink: symlink$2,
    unlink: unlink$2,
    writeFile: writeFile$2,
};

function createFSObject$2({
	sync = false
} = {}) {
	let fs_object = {};

	for (const method_name in methods$2) {
		const method = methods$2[method_name];
		const method_implementation = sync ? method.sync_impl : method.async_impl;

		fs_object[method_name] = (...args) => {

			if (process.env) {
				if ("ANIO_NODE_FS_API_DEBUG" in process.env) {
					console.error(`anio-node-fs-api ${method_name}${sync ? "Sync" : "Async"} with args`, args);
				}
			}

			return method_implementation(fs_implementation, ...args)
		};
	}

	return fs_object
}

function sync_impl$g$1(fs, path, mode) {
	fs.chmodSync(path, mode);
}

async function async_impl$g$1(fs, path, mode) {
	await fs.promises.chmod(path, mode);
}

var chmod$1 = {sync_impl: sync_impl$g$1, async_impl: async_impl$g$1};

function sync_impl$f$1(fs, path, uid, gid) {
	fs.chownSync(path, uid, gid);
}

async function async_impl$f$1(fs, path, uid, gid) {
	await fs.promises.chown(path, uid, gid);
}

var chown$1 = {sync_impl: sync_impl$f$1, async_impl: async_impl$f$1};

function sync_impl$e$1(fs, src, dest) {
	fs.copyFileSync(src, dest);
}

async function async_impl$e$1(fs, src, dest) {
	await fs.promises.copyFile(src, dest);
}

var copyFile$1 = {sync_impl: sync_impl$e$1, async_impl: async_impl$e$1};

function sync_impl$d$1(fs, path) {
	return fs.lstatSync(path)
}

async function async_impl$d$1(fs, path) {
	return await fs.promises.lstat(path)
}

var lstat$1 = {sync_impl: sync_impl$d$1, async_impl: async_impl$d$1};

function sync_impl$c$1(fs, path, options = null) {
	if (options === null) {
		fs.mkdirSync(path);
	} else {
		fs.mkdirSync(path, options);
	}
}

async function async_impl$c$1(fs, path, options = null) {
	if (options === null) {
		await fs.promises.mkdir(path);
	} else {
		await fs.promises.mkdir(path, options);
	}
}

var mkdir$1 = {sync_impl: sync_impl$c$1, async_impl: async_impl$c$1};

function sync_impl$b$1(fs, path) {
	const handle = fs.opendirSync(path);

	return {
		read() {
			return handle.readSync()
		},
		close() {
			handle.closeSync();
		}
	}
}

async function async_impl$b$1(fs, path) {
	const handle = await fs.promises.opendir(path);

	return {
		async read() {
			return await handle.read()
		},
		async close() {
			await handle.close();
		}
	}
}

var openDirectory$1 = {sync_impl: sync_impl$b$1, async_impl: async_impl$b$1};

function sync_impl$a$1(fs, path, flags = "r", mode = 0o666) {
	const fd = fs.openSync(path, flags, mode);

	return {
		read(buffer) {
			return fs.readSync(fd, buffer)
		},
		write(buffer) {
			return fs.writeSync(fd, buffer)
		},
		close() {
			fs.closeSync(fd);
		}
	}
}

async function async_impl$a$1(fs, path, flags = "r", mode = 0o666) {
	const handle = await fs.promises.open(path, flags, mode);

	return {
		async read(buffer) {
			const {bytesRead} = await handle.read(buffer, 0, buffer.byteLength, null);

			return bytesRead
		},
		async write(buffer) {
			const {bytesWritten} = await handle.write(buffer);

			return bytesWritten
		},
		async close() {
			await handle.close();
		}
	}
}

var openFile$1 = {sync_impl: sync_impl$a$1, async_impl: async_impl$a$1};

function sync_impl$9$1(fs, path) {
	return fs.readFileSync(path)
}

async function async_impl$9$1(fs, path) {
	return await fs.promises.readFile(path)
}

var readFile$1 = {sync_impl: sync_impl$9$1, async_impl: async_impl$9$1};

function sync_impl$8$1(fs, path) {
	return fs.readdirSync(path)
}

async function async_impl$8$1(fs, path) {
	return await fs.promises.readdir(path)
}

var readdir$1 = {sync_impl: sync_impl$8$1, async_impl: async_impl$8$1};

function sync_impl$7$1(fs, path) {
	return fs.readlinkSync(path)
}

async function async_impl$7$1(fs, path) {
	return await fs.promises.readlink(path)
}

var readlink$1 = {sync_impl: sync_impl$7$1, async_impl: async_impl$7$1};

function sync_impl$6$1(fs, path) {
	return fs.realpathSync(path)
}

async function async_impl$6$1(fs, path) {
	return await fs.promises.realpath(path)
}

var realpath$1 = {sync_impl: sync_impl$6$1, async_impl: async_impl$6$1};

function sync_impl$5$1(fs, src, dest) {
	fs.renameSync(src, dest);
}

async function async_impl$5$1(fs, src, dest) {
	await fs.promises.rename(src, dest);
}

var rename$1 = {sync_impl: sync_impl$5$1, async_impl: async_impl$5$1};

function sync_impl$4$1(fs, path) {
	fs.rmdirSync(path);
}

async function async_impl$4$1(fs, path) {
	await fs.promises.rmdir(path);
}

var rmdir$1 = {sync_impl: sync_impl$4$1, async_impl: async_impl$4$1};

function sync_impl$3$1(fs, path) {
	return fs.statSync(path)
}

async function async_impl$3$1(fs, path) {
	return await fs.promises.stat(path)
}

var stat$1 = {sync_impl: sync_impl$3$1, async_impl: async_impl$3$1};

function sync_impl$2$1(fs, target, path) {
	fs.symlinkSync(target, path);
}

async function async_impl$2$1(fs, target, path) {
	await fs.promises.symlink(target, path);
}

var symlink$1 = {sync_impl: sync_impl$2$1, async_impl: async_impl$2$1};

function sync_impl$1$1(fs, path) {
	fs.unlinkSync(path);
}

async function async_impl$1$1(fs, path) {
	await fs.promises.unlink(path);
}

var unlink$1 = {sync_impl: sync_impl$1$1, async_impl: async_impl$1$1};

function sync_impl$h$1(fs, path) {
	return fs.writeFileSync(path)
}

async function async_impl$h$1(fs, path) {
	return await fs.promises.writeFile(path)
}

var writeFile$1 = {sync_impl: sync_impl$h$1, async_impl: async_impl$h$1};

/* warning: this file was automatically generated */

var methods$1 = {
    chmod: chmod$1,
    chown: chown$1,
    copyFile: copyFile$1,
    lstat: lstat$1,
    mkdir: mkdir$1,
    openDirectory: openDirectory$1,
    openFile: openFile$1,
    readFile: readFile$1,
    readdir: readdir$1,
    readlink: readlink$1,
    realpath: realpath$1,
    rename: rename$1,
    rmdir: rmdir$1,
    stat: stat$1,
    symlink: symlink$1,
    unlink: unlink$1,
    writeFile: writeFile$1,
};

function createFSObject$1({
	sync = false
} = {}) {
	let fs_object = {};

	for (const method_name in methods$1) {
		const method = methods$1[method_name];
		const method_implementation = sync ? method.sync_impl : method.async_impl;

		fs_object[method_name] = (...args) => {

			if (process.env) {
				if ("ANIO_NODE_FS_API_DEBUG" in process.env) {
					console.error(`anio-node-fs-api ${method_name}${sync ? "Sync" : "Async"} with args`, args);
				}
			}

			return method_implementation(fs_implementation, ...args)
		};
	}

	return fs_object
}

function tryStat$1(fs_object, path, fn = "lstat") {
	try {
		return fs_object[fn](path)
	} catch (error) {
		//
		// ENOENT means no such entry
		//
		if (error.code === "ENOENT") {
			return false
		}

		// re-throw every other error
		throw error
	}
}

function sync_impl$j(fs_object, ...args) {
	const path_to_check = path.join(...args);

	const lstat = tryStat$1(fs_object, path_to_check);

	if (lstat === false) return false

	if (lstat.isSymbolicLink()) {
		const stat = tryStat$1(fs_object, path_to_check, "stat");

		if (stat === false) return "link->broken"
		if (stat.isDirectory()) return "link->dir"

		return "link->file"
	}

	if (lstat.isDirectory()) return "dir"

	return "file"
}

async function tryStat(fs_object, path, fn = "lstat") {
	try {
		return await fs_object[fn](path)
	} catch (error) {
		//
		// ENOENT means no such entry
		//
		if (error.code === "ENOENT") {
			return false
		}

		// re-throw every other error
		throw error
	}
}

async function async_impl$j(fs_object, ...args) {
	const path_to_check = path.join(...args);

	const lstat = await tryStat(fs_object, path_to_check);

	if (lstat === false) return false

	if (lstat.isSymbolicLink()) {
		const stat = await tryStat(fs_object, path_to_check, "stat");

		if (stat === false) return "link->broken"
		if (stat.isDirectory()) return "link->dir"

		return "link->file"
	}

	if (lstat.isDirectory()) return "dir"

	return "file"
}

const async_fs$2 = createFSObject$1({sync: false});
const sync_fs$2 = createFSObject$1({sync: true});

function nodeFsGetPathType(...args) {
	return async_impl$j(async_fs$2, ...args)
}

nodeFsGetPathType.sync = function(...args) {
	return sync_impl$j(sync_fs$2, ...args)
};

function sync_impl$g$3(fs, path, mode) {
	fs.chmodSync(path, mode);
}

async function async_impl$g$3(fs, path, mode) {
	await fs.promises.chmod(path, mode);
}

var chmod$3 = {sync_impl: sync_impl$g$3, async_impl: async_impl$g$3};

function sync_impl$f$3(fs, path, uid, gid) {
	fs.chownSync(path, uid, gid);
}

async function async_impl$f$3(fs, path, uid, gid) {
	await fs.promises.chown(path, uid, gid);
}

var chown$3 = {sync_impl: sync_impl$f$3, async_impl: async_impl$f$3};

function sync_impl$e$3(fs, src, dest) {
	fs.copyFileSync(src, dest);
}

async function async_impl$e$3(fs, src, dest) {
	await fs.promises.copyFile(src, dest);
}

var copyFile$3 = {sync_impl: sync_impl$e$3, async_impl: async_impl$e$3};

function sync_impl$d$3(fs, path) {
	return fs.lstatSync(path)
}

async function async_impl$d$3(fs, path) {
	return await fs.promises.lstat(path)
}

var lstat$3 = {sync_impl: sync_impl$d$3, async_impl: async_impl$d$3};

function sync_impl$c$3(fs, path, options = null) {
	if (options === null) {
		fs.mkdirSync(path);
	} else {
		fs.mkdirSync(path, options);
	}
}

async function async_impl$c$3(fs, path, options = null) {
	if (options === null) {
		await fs.promises.mkdir(path);
	} else {
		await fs.promises.mkdir(path, options);
	}
}

var mkdir$3 = {sync_impl: sync_impl$c$3, async_impl: async_impl$c$3};

function sync_impl$b$3(fs, path) {
	const handle = fs.opendirSync(path);

	return {
		read() {
			return handle.readSync()
		},
		close() {
			handle.closeSync();
		}
	}
}

async function async_impl$b$3(fs, path) {
	const handle = await fs.promises.opendir(path);

	return {
		async read() {
			return await handle.read()
		},
		async close() {
			await handle.close();
		}
	}
}

var openDirectory$3 = {sync_impl: sync_impl$b$3, async_impl: async_impl$b$3};

function sync_impl$a$3(fs, path, flags = "r", mode = 0o666) {
	const fd = fs.openSync(path, flags, mode);

	return {
		read(buffer) {
			return fs.readSync(fd, buffer)
		},
		write(buffer) {
			return fs.writeSync(fd, buffer)
		},
		close() {
			fs.closeSync(fd);
		}
	}
}

async function async_impl$a$3(fs, path, flags = "r", mode = 0o666) {
	const handle = await fs.promises.open(path, flags, mode);

	return {
		async read(buffer) {
			const {bytesRead} = await handle.read(buffer, 0, buffer.byteLength, null);

			return bytesRead
		},
		async write(buffer) {
			const {bytesWritten} = await handle.write(buffer);

			return bytesWritten
		},
		async close() {
			await handle.close();
		}
	}
}

var openFile$3 = {sync_impl: sync_impl$a$3, async_impl: async_impl$a$3};

function sync_impl$9$3(fs, path) {
	return fs.readFileSync(path)
}

async function async_impl$9$3(fs, path) {
	return await fs.promises.readFile(path)
}

var readFile$3 = {sync_impl: sync_impl$9$3, async_impl: async_impl$9$3};

function sync_impl$8$3(fs, path) {
	return fs.readdirSync(path)
}

async function async_impl$8$3(fs, path) {
	return await fs.promises.readdir(path)
}

var readdir$3 = {sync_impl: sync_impl$8$3, async_impl: async_impl$8$3};

function sync_impl$7$3(fs, path) {
	return fs.readlinkSync(path)
}

async function async_impl$7$3(fs, path) {
	return await fs.promises.readlink(path)
}

var readlink$3 = {sync_impl: sync_impl$7$3, async_impl: async_impl$7$3};

function sync_impl$6$3(fs, path) {
	return fs.realpathSync(path)
}

async function async_impl$6$3(fs, path) {
	return await fs.promises.realpath(path)
}

var realpath$3 = {sync_impl: sync_impl$6$3, async_impl: async_impl$6$3};

function sync_impl$5$3(fs, src, dest) {
	fs.renameSync(src, dest);
}

async function async_impl$5$3(fs, src, dest) {
	await fs.promises.rename(src, dest);
}

var rename$3 = {sync_impl: sync_impl$5$3, async_impl: async_impl$5$3};

function sync_impl$4$3(fs, path) {
	fs.rmdirSync(path);
}

async function async_impl$4$3(fs, path) {
	await fs.promises.rmdir(path);
}

var rmdir$3 = {sync_impl: sync_impl$4$3, async_impl: async_impl$4$3};

function sync_impl$3$3(fs, path) {
	return fs.statSync(path)
}

async function async_impl$3$3(fs, path) {
	return await fs.promises.stat(path)
}

var stat$3 = {sync_impl: sync_impl$3$3, async_impl: async_impl$3$3};

function sync_impl$2$3(fs, target, path) {
	fs.symlinkSync(target, path);
}

async function async_impl$2$3(fs, target, path) {
	await fs.promises.symlink(target, path);
}

var symlink$3 = {sync_impl: sync_impl$2$3, async_impl: async_impl$2$3};

function sync_impl$1$3(fs, path) {
	fs.unlinkSync(path);
}

async function async_impl$1$3(fs, path) {
	await fs.promises.unlink(path);
}

var unlink$3 = {sync_impl: sync_impl$1$3, async_impl: async_impl$1$3};

function sync_impl$h$2(fs, path) {
	return fs.writeFileSync(path)
}

async function async_impl$h$2(fs, path) {
	return await fs.promises.writeFile(path)
}

var writeFile$3 = {sync_impl: sync_impl$h$2, async_impl: async_impl$h$2};

/* warning: this file was automatically generated */

var methods$3 = {
    chmod: chmod$3,
    chown: chown$3,
    copyFile: copyFile$3,
    lstat: lstat$3,
    mkdir: mkdir$3,
    openDirectory: openDirectory$3,
    openFile: openFile$3,
    readFile: readFile$3,
    readdir: readdir$3,
    readlink: readlink$3,
    realpath: realpath$3,
    rename: rename$3,
    rmdir: rmdir$3,
    stat: stat$3,
    symlink: symlink$3,
    unlink: unlink$3,
    writeFile: writeFile$3,
};

function createFSObject$3({
	sync = false
} = {}) {
	let fs_object = {};

	for (const method_name in methods$3) {
		const method = methods$3[method_name];
		const method_implementation = sync ? method.sync_impl : method.async_impl;

		fs_object[method_name] = (...args) => {

			if (process.env) {
				if ("ANIO_NODE_FS_API_DEBUG" in process.env) {
					console.error(`anio-node-fs-api ${method_name}${sync ? "Sync" : "Async"} with args`, args);
				}
			}

			return method_implementation(fs_implementation, ...args)
		};
	}

	return fs_object
}

function scandir$1(fs_object, root_dir, relative_entry_dir, options) {
	const entries = fs_object.readdir(
		path.join(root_dir, relative_entry_dir)
	);

	for (const entry of entries) {
		const absolute_path = path.join(root_dir, relative_entry_dir, entry);
		const relative_path = path.join(relative_entry_dir, entry);
		const stats = fs_object.lstat(absolute_path);

		let type = "file";

		if (stats.isSymbolicLink()) {
			type = "link";
		} else if (stats.isDirectory()) {
			type = "dir";
		}

		const handle_current_entry = () => {
			const data = {type, relative_path, absolute_path};

			if (typeof options.callback === "function") {
				options.callback(data);

				return
			}

			options.entries.push(data);
		};

		const recurse = () => {
			if (type !== "dir") return

			scandir$1(fs_object, root_dir, relative_path, options);
		};

		if (options.reverse) {
			recurse();
		}

		handle_current_entry();

		if (!options.reverse) {
			recurse();
		}
	}
}

function sync_impl$i(fs_object, root_dir, {
	callback = null,
	reverse = false
} = {}) {
	let entries = [];
	const options = {callback, reverse, entries};
	const resolved_root_path = fs_object.realpath(root_dir);

	scandir$1(fs_object, resolved_root_path, ".", options);

	return typeof callback === "function" ? null : entries
}

async function scandir(fs_object, root_dir, relative_entry_dir, options) {
	const entries = await fs_object.readdir(
		path.join(root_dir, relative_entry_dir)
	);

	for (const entry of entries) {
		const absolute_path = path.join(root_dir, relative_entry_dir, entry);
		const relative_path = path.join(relative_entry_dir, entry);
		const stats = await fs_object.lstat(absolute_path);

		let type = "file";

		if (stats.isSymbolicLink()) {
			type = "link";
		} else if (stats.isDirectory()) {
			type = "dir";
		}

		const handle_current_entry = async () => {
			const data = {type, relative_path, absolute_path};

			if (typeof options.callback === "function") {
				await options.callback(data);

				return
			}

			options.entries.push(data);
		};

		const recurse = async () => {
			if (type !== "dir") return

			await scandir(fs_object, root_dir, relative_path, options);
		};

		if (options.reverse) {
			await recurse();
		}

		await handle_current_entry();

		if (!options.reverse) {
			await recurse();
		}
	}
}

async function async_impl$i(fs_object, root_dir, {
	callback = null,
	reverse = false
} = {}) {
	let entries = [];
	const options = {callback, reverse, entries};
	const resolved_root_path = await fs_object.realpath(root_dir);

	await scandir(fs_object, resolved_root_path, ".", options);

	return typeof callback === "function" ? null : entries
}

const async_fs$1 = createFSObject$3({sync: false});
const sync_fs$1 = createFSObject$3({sync: true});

function nodeFsScandir(root_dir, options = {}) {
	return async_impl$i(async_fs$1, root_dir, options)
}

nodeFsScandir.sync = function(root_dir, options = {}) {
	return sync_impl$i(sync_fs$1, root_dir, options)
};

function removeSymbolicLink$1(fs_object, src) {
	fs_object.unlink(src);
}

function removeFile$1(fs_object, src) {
	fs_object.unlink(src);
}

function removeDirectory$1(fs_object, src) {
	nodeFsScandir.sync(src, {
		callback({type, relative_path, absolute_path}) {
			const args = [fs_object, absolute_path];

			if (type === "link") {
				removeSymbolicLink$1(...args);
			} else if (type === "dir") {
				removeDirectory$1(...args);
			} else {
				removeFile$1(...args);
			}
		},
		reverse: true
	});

	fs_object.rmdir(src);
}

const remove_map$1 = {
	"link->dir": removeSymbolicLink$1,
	"link->file": removeSymbolicLink$1,
	"link->broken": removeSymbolicLink$1,
	"file": removeFile$1,
	"dir": removeDirectory$1
};

function sync_impl$l(fs_object, src) {
	const path_type = nodeFsGetPathType.sync(src);

	// 'src' does not exist
	if (path_type === false) {
		return
	}

	if (!(path_type in remove_map$1)) {
		throw new Error(`Don't know how to remove path of type '${path_type}'.`)
	}

	const remove_fn = remove_map$1[path_type];

	return remove_fn(fs_object, src)
}

async function removeSymbolicLink(fs_object, src) {
	await fs_object.unlink(src);
}

async function removeFile(fs_object, src) {
	await fs_object.unlink(src);
}

async function removeDirectory(fs_object, src) {
	await nodeFsScandir(src, {
		async callback({type, relative_path, absolute_path}) {
			const args = [fs_object, absolute_path];

			if (type === "link") {
				await removeSymbolicLink(...args);
			} else if (type === "dir") {
				await removeDirectory(...args);
			} else {
				await removeFile(...args);
			}
		},
		reverse: true
	});

	await fs_object.rmdir(src);
}

const remove_map = {
	"link->dir": removeSymbolicLink,
	"link->file": removeSymbolicLink,
	"link->broken": removeSymbolicLink,
	"file": removeFile,
	"dir": removeDirectory
};

async function async_impl$l(fs_object, src) {
	const path_type = await nodeFsGetPathType(src);

	// 'src' does not exist
	if (path_type === false) {
		return
	}

	if (!(path_type in remove_map)) {
		throw new Error(`Don't know how to remove path of type '${path_type}'.`)
	}

	const remove_fn = remove_map[path_type];

	return await remove_fn(fs_object, src)
}

const async_fs$3 = createFSObject$2({sync: false});
const sync_fs$3 = createFSObject$2({sync: true});

function nodeFsRemove(src, dest) {
	return async_impl$l(async_fs$3, src)
}

nodeFsRemove.sync = function(src, dest) {
	return sync_impl$l(sync_fs$3, src)
};

async function remove(...args) {
	return await nodeFsRemove(...args)
}

remove.sync = function(...args) {
	return nodeFsRemove.sync(...args)
};

function sync_impl$g(fs, path, mode) {
	fs.chmodSync(path, mode);
}

async function async_impl$g(fs, path, mode) {
	await fs.promises.chmod(path, mode);
}

var chmod = {sync_impl: sync_impl$g, async_impl: async_impl$g};

function sync_impl$f(fs, path, uid, gid) {
	fs.chownSync(path, uid, gid);
}

async function async_impl$f(fs, path, uid, gid) {
	await fs.promises.chown(path, uid, gid);
}

var chown = {sync_impl: sync_impl$f, async_impl: async_impl$f};

function sync_impl$e(fs, src, dest) {
	fs.copyFileSync(src, dest);
}

async function async_impl$e(fs, src, dest) {
	await fs.promises.copyFile(src, dest);
}

var copyFile = {sync_impl: sync_impl$e, async_impl: async_impl$e};

function sync_impl$d(fs, path) {
	return fs.lstatSync(path)
}

async function async_impl$d(fs, path) {
	return await fs.promises.lstat(path)
}

var lstat = {sync_impl: sync_impl$d, async_impl: async_impl$d};

function sync_impl$c(fs, path, options = null) {
	if (options === null) {
		fs.mkdirSync(path);
	} else {
		fs.mkdirSync(path, options);
	}
}

async function async_impl$c(fs, path, options = null) {
	if (options === null) {
		await fs.promises.mkdir(path);
	} else {
		await fs.promises.mkdir(path, options);
	}
}

var mkdir = {sync_impl: sync_impl$c, async_impl: async_impl$c};

function sync_impl$b(fs, path) {
	const handle = fs.opendirSync(path);

	return {
		read() {
			return handle.readSync()
		},
		close() {
			handle.closeSync();
		}
	}
}

async function async_impl$b(fs, path) {
	const handle = await fs.promises.opendir(path);

	return {
		async read() {
			return await handle.read()
		},
		async close() {
			await handle.close();
		}
	}
}

var openDirectory = {sync_impl: sync_impl$b, async_impl: async_impl$b};

function sync_impl$a(fs, path, flags = "r", mode = 0o666) {
	const fd = fs.openSync(path, flags, mode);

	return {
		read(buffer) {
			return fs.readSync(fd, buffer)
		},
		write(buffer) {
			return fs.writeSync(fd, buffer)
		},
		close() {
			fs.closeSync(fd);
		}
	}
}

async function async_impl$a(fs, path, flags = "r", mode = 0o666) {
	const handle = await fs.promises.open(path, flags, mode);

	return {
		async read(buffer) {
			const {bytesRead} = await handle.read(buffer, 0, buffer.byteLength, null);

			return bytesRead
		},
		async write(buffer) {
			const {bytesWritten} = await handle.write(buffer);

			return bytesWritten
		},
		async close() {
			await handle.close();
		}
	}
}

var openFile = {sync_impl: sync_impl$a, async_impl: async_impl$a};

function sync_impl$9(fs, path) {
	return fs.readFileSync(path)
}

async function async_impl$9(fs, path) {
	return await fs.promises.readFile(path)
}

var readFile = {sync_impl: sync_impl$9, async_impl: async_impl$9};

function sync_impl$8(fs, path) {
	return fs.readdirSync(path)
}

async function async_impl$8(fs, path) {
	return await fs.promises.readdir(path)
}

var readdir = {sync_impl: sync_impl$8, async_impl: async_impl$8};

function sync_impl$7(fs, path) {
	return fs.readlinkSync(path)
}

async function async_impl$7(fs, path) {
	return await fs.promises.readlink(path)
}

var readlink = {sync_impl: sync_impl$7, async_impl: async_impl$7};

function sync_impl$6(fs, path) {
	return fs.realpathSync(path)
}

async function async_impl$6(fs, path) {
	return await fs.promises.realpath(path)
}

var realpath = {sync_impl: sync_impl$6, async_impl: async_impl$6};

function sync_impl$5(fs, src, dest) {
	fs.renameSync(src, dest);
}

async function async_impl$5(fs, src, dest) {
	await fs.promises.rename(src, dest);
}

var rename = {sync_impl: sync_impl$5, async_impl: async_impl$5};

function sync_impl$4(fs, path) {
	fs.rmdirSync(path);
}

async function async_impl$4(fs, path) {
	await fs.promises.rmdir(path);
}

var rmdir = {sync_impl: sync_impl$4, async_impl: async_impl$4};

function sync_impl$3(fs, path) {
	return fs.statSync(path)
}

async function async_impl$3(fs, path) {
	return await fs.promises.stat(path)
}

var stat = {sync_impl: sync_impl$3, async_impl: async_impl$3};

function sync_impl$2(fs, target, path) {
	fs.symlinkSync(target, path);
}

async function async_impl$2(fs, target, path) {
	await fs.promises.symlink(target, path);
}

var symlink = {sync_impl: sync_impl$2, async_impl: async_impl$2};

function sync_impl$1(fs, path) {
	fs.unlinkSync(path);
}

async function async_impl$1(fs, path) {
	await fs.promises.unlink(path);
}

var unlink = {sync_impl: sync_impl$1, async_impl: async_impl$1};

function sync_impl$h(fs, path, data) {
	return fs.writeFileSync(path, data)
}

async function async_impl$h(fs, path, data) {
	return await fs.promises.writeFile(path, data)
}

var writeFile = {sync_impl: sync_impl$h, async_impl: async_impl$h};

/* warning: this file was automatically generated */

var methods = {
    chmod,
    chown,
    copyFile,
    lstat,
    mkdir,
    openDirectory,
    openFile,
    readFile,
    readdir,
    readlink,
    realpath,
    rename,
    rmdir,
    stat,
    symlink,
    unlink,
    writeFile,
};

function createFSObject({
	sync = false
} = {}) {
	let fs_object = {};

	for (const method_name in methods) {
		const method = methods[method_name];
		const method_implementation = sync ? method.sync_impl : method.async_impl;

		fs_object[method_name] = (...args) => {

			if (process.env) {
				if ("ANIO_NODE_FS_API_DEBUG" in process.env) {
					console.error(`anio-node-fs-api ${method_name}${sync ? "Sync" : "Async"} with args`, args);
				}
			}

			return method_implementation(fs_implementation, ...args)
		};
	}

	return fs_object
}

function createRandomIdentifier(len) {
	let str = "";
	let bytes = [];

	if (typeof crypto === "object" && typeof crypto.getRandomValues === "function") {
		bytes = new Uint8Array(len);

		crypto.getRandomValues(bytes);
	} else {
		for (let i = 0; i < len; ++i) {
			// will never exceed 255
			bytes.push(Math.floor(Math.random() * 256));
		}
	}

	for (let i = 0; i < len; ++i) {
		str += (bytes[i]).toString(16).slice(0, 1);
	}

	// sanity check output
	if (str.length !== len) {
		throw new Error(`createRandomIdentifier(${len}) failed for unknown reasons.`)
	}

	return str
}

function sync_impl(fs_object, file_path, data) {
	const random_str = createRandomIdentifier(16);

	const parent_dir = path.dirname(file_path);
	const file_name = path.basename(file_path);
	const tmp_name = file_name + `.tmp${random_str}`;
	const tmp_path = path.join(parent_dir, tmp_name);

	fs_object.writeFile(tmp_path, data);
	fs_object.rename(tmp_path, file_path);

	return data.length
}

async function async_impl(fs_object, file_path, data) {
	const random_str = createRandomIdentifier(16);

	const parent_dir = path.dirname(file_path);
	const file_name = path.basename(file_path);
	const tmp_name = file_name + `.tmp${random_str}`;
	const tmp_path = path.join(parent_dir, tmp_name);

	await fs_object.writeFile(tmp_path, data);
	await fs_object.rename(tmp_path, file_path);

	return data.length
}

const async_fs = createFSObject({sync: false});
const sync_fs = createFSObject({sync: true});

async function nodeFsWriteAtomicFile(file_path, data) {
	return await async_impl(async_fs, file_path, data)
}

nodeFsWriteAtomicFile.sync = function(file_path, data) {
	return sync_impl(sync_fs, file_path, data)
};

async function writeAtomicFile(...args) {
	return await nodeFsWriteAtomicFile(...args)
}

writeAtomicFile.sync = function(...args) {
	return nodeFsWriteAtomicFile.sync(...args)
};

/* warning: this file was automatically generated */

var index = {
    copy,
    generateTemporaryPathName,
    hashFile,
    isDirectory,
    isFile,
    isRegularDirectory,
    isRegularFile,
    isSymbolicLink,
    remove,
    writeAtomicFile,
};

export { index as default };
